@@ -145,6 +145,13 @@ list_t *channel_name_list(struct channel *c)
 	return ret;
 }
 
+char *link_name(struct link_any *l)
+{
+	if (LINK(l))
+		return LINK(l)->name ? LINK(l)->name : "(null)";
+	return "*connecting*";
+}
+
 static int irc_001(struct link_server *server, struct line *line)
 {
 	(void)line;
@@ -370,7 +377,7 @@ int irc_dispatch_server(bip_t *bip, struct link_server *server,
 	} else if (irc_line_elem_equals(line, 0, "PONG")) {
 		/* not all server reply with PONG <servername> <our string>
 		 * so we blindly assume the PONG is ours. */
-		if (irc_line_count(line) == 3) {
+		if (irc_line_count(line) == 2 || irc_line_count(line) == 3) {
 			if (server->laginit_ts != -1) {
 				irc_compute_lag(server);
 				irc_lag_init(server);
@@ -1106,7 +1113,7 @@ static int irc_dispatch_client(bip_t *bip, struct link_client *ic,
 	if (irc_line_elem_equals(line, 0, "PING")) {
 		if (!irc_line_includes(line, 1))
 			return ERR_PROTOCOL;
-		WRITE_LINE1(CONN(ic), LINK(ic)->name, "PONG",
+		WRITE_LINE1(CONN(ic), link_name((struct link_any *)ic), "PONG",
 				irc_line_elem(line, 1));
 		r = OK_FORGET;
 	} else if (LINK(ic)->s_state != IRCS_CONNECTED) {
@@ -1715,9 +1722,10 @@ static int irc_mode(struct link_server *server, struct line *line)
 		case 'e':
 		case 'q':
 		case 'I':
-			if (!irc_line_includes(line, cur_arg + 3))
-				return ERR_PROTOCOL;
-			cur_arg++;
+			/* Sucky way to try to deal with all the different mode
+			 * out there... */
+			if (irc_line_includes(line, cur_arg + 3))
+				cur_arg++;
 			break;
 		default:
 			break;
@@ -1960,6 +1968,9 @@ static void irc_server_startup(struct link_server *ircs)
 	char *nick;
 	char *username, *realname;
 
+	/* lower the token number as freenode hates fast login */
+        CONN(ircs)->token = 1;
+
 	if (LINK(ircs)->s_password)
 		WRITE_LINE1(CONN(ircs), NULL, "PASS", LINK(ircs)->s_password);
 
@@ -2445,7 +2456,7 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 	if (err) {
 		if (TYPE(lc) == IRC_TYPE_SERVER) {
 			mylog(LOG_ERROR, "[%s] read_lines error, closing...",
-					LINK(lc)->name);
+					link_name(lc));
 			irc_server_shutdown(LINK(lc)->l_server);
 		} else {
 			mylog(LOG_ERROR, "client read_lines error, closing...");
@@ -2467,7 +2478,7 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 		line = irc_line_new_from_string(line_s);
 		if (!line) {
 			mylog(LOG_ERROR, "[%s] Error in protocol, closing...",
-					LINK(lc)->name);
+					link_name(lc));
 			free(line_s);
 			goto prot_err_lines;
 		}
@@ -2477,7 +2488,7 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 		free(line_s);
 		if (r == ERR_PROTOCOL) {
 			mylog(LOG_ERROR, "[%s] Error in protocol, closing...",
-					LINK(lc)->name);
+					link_name(lc));
 			goto prot_err_lines;
 		}
 		if (r == ERR_AUTH)
