@@ -11,6 +11,8 @@
  * See the file "COPYING" for the exact licensing terms.
  */
 
+#define _XOPEN_SOURCE 500
+
 #include "config.h"
 #include "log.h"
 #include "irc.h"
@@ -954,7 +956,6 @@ char *log_beautify(log_t *logdata, const char *buf, const char *storename,
 static time_t compute_time(const char *buf)
 {
 	struct tm tm;
-	int err;
 	time_t tv;
 
 	/* this is to fill tm_isdst to current tm, expect brokennes when dst
@@ -962,11 +963,8 @@ static time_t compute_time(const char *buf)
 	time(&tv);
 	tm = *localtime(&tv);
 
-	err = sscanf(buf, "%2d-%2d-%4d %2d:%2d:%2d", &tm.tm_mday, &tm.tm_mon,
-			&tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
-	if (err != 6)
+	if (strptime(buf, "%d-%m-%Y %H:%M:%S", &tm) == NULL)
 		return (time_t)-1;
-	tm.tm_year -= 1900;
 	return mktime(&tm);
 }
 
@@ -1312,9 +1310,9 @@ static list_t *log_backread_hours(log_t *log, const char *storename,
 	list_iterator_t file_it;
 
 	gettimeofday(&tv, NULL);
-	if (tv.tv_sec <= 3600 * 24 * hours)
+	if (tv.tv_sec <= 3600 * hours)
 		return NULL;
-	blstarttime = tv.tv_sec - 3600 * 24 * hours;
+	blstarttime = tv.tv_sec - 3600 * hours;
 
 	store = hash_get(&log->logfgs, storename);
 
